; Copyright 2000, by Michael Vincent & Robin Kay. All rights reserved.
;
;
.nolist    
#include "tse.inc"
.list

.org    saferam2-3
.db     $BB,$6D
 ret

start:
 jp verinfo     ; Lib 0, Get functions supported
 jp rand        ; Lib 1, Random number generator
 jp sprite      ; Lib 2, XOR a sprite
 jp largeSprite ; Lib 3, XOR a large sprite
 jp getpix      ; Lib 4, Get pixel
 jp decomp      ; Lib 5, Decompress
 jp fastcopy    ; Lib 6, Fast Copy
 jp findvar     ; Lib 7, Find Variable

verinfo:
 ld hl, %0000000011111111
 ret

;---------= Random number generator =---------
; input b=upper bound
; ouput a=answer 0<=a<b
; all registers are preserved except: af and bc
;
rand:
 push    hl
 push    de
 ld      hl,(data)
 ld      a,r
 ld      d,a
 ld      e,(hl)
 add     hl,de
 add     a,l
 xor     h
 ld      (data),hl
 ld      hl,0
 ld      e,a
 ld      d,h
randl:
 add     hl,de
 djnz    randl
 ld      a,h
 pop     de
 pop     hl
 ret

;---------= XOR a sprite =---------
; b=size of sprite
; l=yc
; a=xc
; ix holds pointer
;
sprite:
 ld      e,l
 ld      h,$00
 ld      d,h
 add     hl,de
 add     hl,de
 add     hl,hl
 add     hl,hl
 ld      e,a
 and     $07
 ld      c,a
 srl     e
 srl     e
 srl     e
 add     hl,de
 ld      de,plotsscreen
 add     hl,de
sl1:
 ld      d,(ix)
 ld      e,$00
 ld      a,c
 or      a
 jr      z,sl3
sl2:
 srl     d
 rr      e
 dec     a
 jr      nz,sl2
sl3:
 ld      a,(hl)
 xor     d
 ld      (hl),a
 inc     hl
 ld      a,(hl)
 xor     e
 ld      (hl),a
 ld      de,$0B
 add     hl,de
 inc     ix
 djnz    sl1
 ret

;---------= Large Sprite =---------
; Draws a Large sprite.
;Input:	ix->sprite
;	a=x
;	l=y
;	b=height	(in pixels)
;	c=width		(in bytes, e.g. 2 would be 16)
;Output: nothing
; All registers are destroyed except bc', de', hl'
;
largeSprite:
 di
 ex      af,af'
 ld      a,c
 push    af
 ex      af,af'
 ld      e,l
 ld      h,$00
 ld      d,h
 add     hl,de
 add     hl,de
 add     hl,hl
 add     hl,hl
 ld      e,a
 and     $07
 ld      c,a
 srl     e
 srl     e
 srl     e
 add     hl,de
 ld      de,plotsscreen
 add     hl,de
lsl0:
 push    hl
lsl1:
 ld      d,(ix)
 ld      e,$00
 ld      a,c
 or      a
 jr      z,lsl3
lsl2:
 srl     d
 rr      e
 dec     a
 jr      nz,lsl2
lsl3:
 ld      a,(hl)
 xor     d
 ld      (hl),a
 inc     hl
 ld      a,(hl)
 xor     e
 ld      (hl),a
 inc     ix
 ex      af,af'
 dec     a
 push    af
 ex      af,af'
 pop     af
 jr      nz,lsl1
 pop     hl
 pop     af
 push    af
 ex      af,af'
 ld      de,$0C
 add     hl,de
 djnz    lsl0
 pop     af
 ei
 ret

;---------= Get location of a pixel =---------
; input:	e=y coordinate
;		a=x coordinate
; output:	a holds data for pixel (e.g. %00100000)
;               hl->byte where pixel is on the plotsscreen
;
getpix:
 ld      d,$00
 ld      h,d
 ld      l,e
 add     hl,de
 add     hl,de
 add     hl,hl
 add     hl,hl
 ; ION wants an offset?
 ld      de,plotsscreen
 add     hl,de
getbit:
 ld      b,$00
 ld      c,a
 and     %00000111
 srl     c
 srl     c
 srl     c
 add     hl,bc
 ld      b,a
 inc     b
 ld      a,%00000001
gblp:
 rrca
 djnz    gblp
 ret

;---------= Decompress =---------
; input:
;	hl->compressed data
;	de->place to load data
;	b=length of compressed data
;	c=compression factor (1, 3, or 15)
; output:
;	level is decompressed
;	hl->next byte of compressed data
;
decomp:
 di
decompLoop:
 push    bc
 ld      a,(hl)
 ex      af,af'
 ld      a,c
 ld      b,8
 cp      1
 jr      z,dcmp1
 ld      b,4
 cp      3
 jr      z,dcmp1
 ld      b,2
dcmp1:
 push    bc
 ld      a,c
 ld      b,1
 cp      1
 jr      z,dcmp2
 inc     b
 cp      3
 jr      z,dcmp2
 ld      b,4
dcmp2:
 ex      af,af'
dcmp3:
 rlca
 djnz    dcmp3
 ld      b,a
 ex      af,af'
 ld      a,b
 and     c
 ld      (de),a
 inc     de
 pop     bc
 djnz    dcmp1
 inc     hl
 pop     bc
 djnz    decompLoop
 ei
 ret

;---------= Fast Copy =---------
;Input:	nothing
;Output:graph buffer is copied to the screen
;
fastCopy:
 di
 ld      a,$80
 out     ($10),a
 ld      hl,plotsscreen-12-(-(12*64)+1)
 ld      a,$20
 ld      c,a
 inc hl
 dec hl
fastCopyAgain:
 ld      b,64
 inc     c
 ld      de,-(12*64)+1
 out     ($10),a
 add     hl,de
 ld      de,0ah
fastCopyLoop:
 add     hl,de
 inc     hl
 inc     hl
 inc     de
 ld      a,(hl)
 out     ($11),a
 dec     de
 djnz    fastCopyLoop
 ld      a,c
 cp      $2B+1
 jr      nz,fastCopyAgain
 ei
 ret

;---------= Find Variable =---------
;Input: a=variable number starting at 1
;Output: op1=variable name
;        zf =set if found
;
findVar:
 ld (data), a
 ld hl, (ProgPtr)
 ld bc, (pTemp)
 ld d, 0
 call searchLoop
 ld a, (data)
 cp 0
 ret

searchLoop:
 ; Check to see if finished reading
 or a
 sbc hl, bc
 ret c
 ret z
 add hl, bc

 ; Load and mask object type
 ld a, (hl)
 and 1Fh

 ; Get address of NL 
 ld e, 6
 or a
 sbc hl, de

 ; Found program?
 cp ProtProgObj
 call z, foundVar

 ; Advance to next entry
 ld e, (hl)
 inc e
 or a
 sbc hl, de
 jr searchloop

; Variable found
foundVar:
 ; Decrease index
 ld a, (data)
 dec a
 ld (data), a
 cp 0
 ret nz

 ; Prefix op1 with ProtProgObj
 ld a, ProtProgObj   
 ld (op1), a

 ; Load op1 with variable name
 ld de, op1
 ld b, (hl)          
loopLoadReverseName:
 dec hl
 inc de
 ld a, (hl)
 ld (de), a
 djnz loopLoadReverseName

 ; Terminate op1 with null
 inc de           
 xor a 
 ld (de), a

 ; Pop off return address and ret
 pop hl
 ret
; Data for library routines
; 
data:
.end
